import functools
from datetime import datetime
from datetime import timedelta
from datetime import timezone
from typing import Callable
from typing import cast
from typing import Optional
from typing import Protocol

import pandas as pd

from .compat import ParamSpec
from .compat import TypeAlias

P = ParamSpec('P')

PandasFunc: TypeAlias = Callable[P, pd.DataFrame]


class LRUCacheWrapper(Protocol[P]):
    lifetime: timedelta
    expiration: datetime

    def __call__(self, *args: P.args, **kwargs: P.kwargs) -> pd.DataFrame:
        pass

    def clear(self):
        pass


def lru_cache(seconds: Optional[int] = None, maxsize: Optional[int] = None, typed: bool = True):
    """
    装饰器工厂，生成一个装饰器，该装饰器在内存中缓存熊猫数据帧（具有一定的过期时间），以便将来更快地检索.

    :param seconds: 保留缓存的秒数
    :param maxsize: 要存储在缓存中的最大项目数
    :param typed: 不同类型的参数是否将单独缓存
    :return: 在内存中缓存 pd.DataFrame 的装饰器（含过期时间）.
    """

    def decorator(func: PandasFunc) -> PandasFunc:
        lru_func = cast(LRUCacheWrapper, functools.lru_cache(maxsize=maxsize, typed=typed)(func))

        if not seconds:
            return lru_func

        lru_func.lifetime = timedelta(seconds=seconds)
        lru_func.expiration = datetime.now(timezone.utc) + lru_func.lifetime

        @functools.wraps(func)
        def retrieve_cache(*args: P.args, **kwargs: P.kwargs) -> pd.DataFrame:
            if datetime.now(timezone.utc) >= lru_func.expiration:
                lru_func.clear()
                lru_func.expiration = datetime.now(timezone.utc) + lru_func.lifetime

            return lru_func(*args, **kwargs)

        return cast(PandasFunc, retrieve_cache)

    return decorator



import functools
import os
import pathlib
import time
from typing import Callable
from typing import Optional
from typing import Union

import pandas as pd

from ..exceptions import FileNeedRefresh
from .compat import ParamSpec
from .compat import TypeAlias

P = ParamSpec('P')

PathLike: TypeAlias = Union[str, pathlib.Path]


def file_cache(filepath: PathLike, refresh_time: Optional[float] = None):
    def decorator(func: Callable[P, pd.DataFrame]):
        @functools.wraps(func)
        def retrieve_cache(*args: P.args, **kwargs: P.kwargs):
            try:
                if refresh_time is not None and os.path.getmtime(filepath) + int(refresh_time) < time.time():
                    raise FileNeedRefresh(f'{filepath} 太旧，需要刷新')
                dataframe: pd.DataFrame = pd.read_pickle(filepath)
            except (FileNotFoundError, EOFError):
                pathlib.Path(filepath).parent.mkdir(exist_ok=True, parents=True)

                dataframe = func(*args, **kwargs)
                dataframe.to_pickle(filepath)

            return dataframe

        return retrieve_cache

    return decorator



import asyncio
import glob
from functools import partial
from pathlib import Path

import pandas as pd

from mootdx.logger import logger


def txt2csv(infile: str, outfile: str = None) -> pd.DataFrame:
    """通达信导出文件转换为 Pandas 可用的 csv 文件

    :param infile: 通达信导出的 txt 文件路径
    :param outfile: 转换后的目标 csv 文件路径
    """

    try:
        names = ['date', 'open', 'high', 'low', 'close', 'volume', 'amount']
        df = pd.read_csv(infile, names=names, header=2, skipfooter=1, index_col='date', engine='python', encoding='gbk')

        # 传参 outfile 目录存在则写文件
        outfile = outfile if outfile else infile.replace('.txt', '.csv')
        Path(outfile).parent.is_dir() and df.to_csv(outfile)

        return df
    except FileNotFoundError as ex:
        logger.error(f'输入文件不存在: {infile}')
        return pd.DataFrame(None)
    except (ValueError, TypeError) as ex:
        logger.error(f'无法解析输入文件: {infile}')
        return pd.DataFrame(None)


async def covert(src, dst):
    return await asyncio.get_event_loop().run_in_executor(None, partial(txt2csv, infile=src, outfile=dst))


def batch(src, dst):
    """批量转换通达信导出文件

    :param src: 来源目录
    :param dst: 目标目录
    """

    tasks = []
    event = asyncio.get_event_loop()

    # 分配任务
    for x in glob.glob1(src, '*.txt'):
        src_ = str(Path(src, x))
        dst_ = src_.replace('.txt', '.csv')

        task = event.create_task(covert(src=src_, dst=dst_))
        tasks.append(task)

    # 执行任务
    event.run_until_complete(asyncio.wait(tasks))


__all__ = ('txt2csv', 'batch')



# @Author  : BoPo
# @Time    : 2021/10/11 17:28
# @Function:
import json
from pathlib import Path

import httpx
import pandas as pd
from tenacity import retry
from tenacity import stop_after_attempt
from tenacity import wait_fixed

from mootdx import get_config_path
from mootdx.cache import file_cache
from mootdx.consts import return_last_value
from mootdx.quotes import Quotes


@retry(wait=wait_fixed(2), retry_error_callback=return_last_value, stop=stop_after_attempt(5))
def fq_factor(method: str, symbol: str) -> pd.DataFrame:
    zh_sina_a_stock_hfq_url = 'https://finance.sina.com.cn/realstock/company/{}/hfq.js'
    zh_sina_a_stock_qfq_url = 'https://finance.sina.com.cn/realstock/company/{}/qfq.js'

    client = httpx.Client(verify=False)

    if method == 'hfq':
        res = client.get(zh_sina_a_stock_hfq_url.format(symbol))
        hfq_factor_df = pd.DataFrame(json.loads(res.text.split('=')[1].split('\n')[0])['data'])

        if hfq_factor_df.shape[0] == 0:
            raise ValueError('sina hfq factor not available')

        hfq_factor_df.columns = ['date', 'hfq_factor']
        hfq_factor_df.index = pd.to_datetime(hfq_factor_df.date)

        del hfq_factor_df['date']

        hfq_factor_df.reset_index(inplace=True)
        # hfq_factor_df = hfq_factor_df.set_index('date')

        return hfq_factor_df
    else:
        res = client.get(zh_sina_a_stock_qfq_url.format(symbol))
        qfq_factor_df = pd.DataFrame(json.loads(res.text.split('=')[1].split('\n')[0])['data'])

        if qfq_factor_df.shape[0] == 0:
            raise ValueError('sina hfq factor not available')

        qfq_factor_df.columns = ['date', 'qfq_factor']
        qfq_factor_df.index = pd.to_datetime(qfq_factor_df.date)

        del qfq_factor_df['date']

        qfq_factor_df.reset_index(inplace=True)
        # qfq_factor_df = qfq_factor_df.set_index('date')

        return qfq_factor_df


def get_xdxr(symbol):
    @file_cache(filepath=Path(get_config_path(f'xdxr/{symbol}.plk')), refresh_time=3600 * 24)
    def _xdxr(symbol):
        xdxr = Quotes.factory('std').xdxr(symbol=symbol)

        if xdxr.empty:
            return xdxr

        xdxr['code'] = symbol
        xdxr['date'] = pd.to_datetime(xdxr[['year', 'month', 'day']], utc=False)

        return xdxr.set_index(['date'])

    return _xdxr(symbol)


def to_adjust(temp_df, symbol=None, adjust=None):
    from mootdx.tools.reversion import reversion
    return reversion(temp_df, get_xdxr(symbol), adjust)


def to_adjust2(temp_df, symbol=None, adjust=None):
    # zh_sina_a_stock_hfq_url = "https://finance.sina.com.cn/realstock/company/{}/hfq.js"
    # zh_sina_a_stock_qfq_url = "https://finance.sina.com.cn/realstock/company/{}/qfq.js"

    temp_df['volume'] = temp_df['vol']
    temp_df['date'] = pd.to_datetime(temp_df[['year', 'month', 'day']])
    temp_df = temp_df.set_index('date')

    if adjust == 'hfq':
        # res = requests.get(zh_sina_a_stock_hfq_url.format(symbol))
        # hfq_factor_df = pd.DataFrame(eval(res.text.split("=")[1].split("\n")[0])["data"])
        # hfq_factor_df.columns = ["date", "hfq_factor"]
        # hfq_factor_df.index = pd.to_datetime(hfq_factor_df.date)

        hfq_factor_df = fq_factor(symbol=symbol, method=adjust)
        del hfq_factor_df['date']

        temp_df = pd.merge(temp_df, hfq_factor_df, left_index=True, right_index=True, how='outer')
        temp_df.fillna(method='ffill', inplace=True)
        temp_df = temp_df.astype(float)
        temp_df.dropna(inplace=True)
        temp_df.drop_duplicates(subset=['open', 'high', 'low', 'close', 'volume'], inplace=True)

        for field in ['open', 'high', 'low', 'close']:
            temp_df[field] = temp_df[field] * temp_df['hfq_factor']

        temp_df = temp_df.iloc[:, :-1]
        # temp_df = temp_df[start_date:end_date]

        temp_df.dropna(inplace=True)
        temp_df.reset_index(inplace=True)

        return temp_df

    if adjust == 'qfq':
        # res = requests.get(zh_sina_a_stock_qfq_url.format(symbol))
        # qfq_factor_df = pd.DataFrame(eval(res.text.split("=")[1].split("\n")[0])["data"])
        # qfq_factor_df.columns = ["date", "qfq_factor"]
        # qfq_factor_df.index = pd.to_datetime(qfq_factor_df.date)
        qfq_factor_df = fq_factor(symbol=symbol, method=adjust)
        qfq_factor_df = qfq_factor_df.set_index('date')
        # del qfq_factor_df["date"]

        temp_df = pd.merge(temp_df, qfq_factor_df, left_index=True, right_index=True, how='outer')
        temp_df.fillna(method='ffill', inplace=True)

        # temp_df = temp_df.astype(float)

        for field in ['open', 'high', 'low', 'close', 'volume', 'qfq_factor']:
            temp_df[field] = temp_df[field].astype(float)

        temp_df.dropna(inplace=True)
        temp_df.drop_duplicates(subset=['open', 'high', 'low', 'close', 'volume'], inplace=True)

        for field in ['open', 'high', 'low', 'close']:
            temp_df[field] = temp_df[field] / temp_df['qfq_factor']

        temp_df = temp_df.iloc[:, :-1]
        temp_df.dropna(inplace=True)
        temp_df.reset_index(inplace=True)

        return temp_df

    return temp_df



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['mootdx',
 'mootdx.cache',
 'mootdx.contrib',
 'mootdx.financial',
 'mootdx.tools',
 'mootdx.utils']

package_data = \
{'': ['*']}

install_requires = \
['httpx>=0.23.1,<0.24.0',
 'pandas>=1.5.2,<2.0.0',
 'tdxpy>=0.1.10,<0.2.0',
 'tenacity>=8.1.0,<9.0.0',
 'tqdm>=4.65.0,<5.0.0',
 'typing-extensions>=4.5.0,<5.0.0']

extras_require = \
{'all': ['py-mini-racer>=0.6.0,<0.7.0',
         'prettytable>=3.5.0,<4.0.0',
         'click>=8.1.3,<9.0.0'],
 'cli': ['prettytable>=3.5.0,<4.0.0', 'click>=8.1.3,<9.0.0'],
 'fetch': ['py-mini-racer>=0.6.0,<0.7.0']}

entry_points = \
{'console_scripts': ['mootdx = mootdx.__main__:entry']}

setup_kwargs = {
    'name': 'mootdx',
    'version': '0.10.7',
    'description': '通达信数据读取接口.',
    'long_description': "通达信数据读取接口\n==================\n\n[![image](https://badge.fury.io/py/mootdx.svg)](http://badge.fury.io/py/mootdx)\n[![image](https://img.shields.io/travis/bopo/mootdx.svg)](https://travis-ci.org/mootdx/mootdx)\n[![Documentation Status](https://readthedocs.org/projects/mootdx/badge/?version=latest)](https://mootdx.readthedocs.io/zh/latest/?badge=latest)\n[![Updates](https://pyup.io/repos/github/mootdx/mootdx/shield.svg)](https://pyup.io/repos/github/mootdx/mootdx/)\n\n如果喜欢本项目可以在右上角给颗⭐！你的支持是我最大的动力😎！\n\n郑重声明: 本项目只作学习交流, 不得用于任何商业目的.\n\n-   开源协议: MIT license\n-   在线文档: <https://www.mootdx.com>\n-   国内镜像: <https://gitee.com/ibopo/mootdx>\n-   项目仓库: <https://github.com/mootdx/mootdx>\n-   问题交流: <https://github.com/mootdx/mootdx/issues>\n\n版本更新(倒序)\n--------------\n\n版本更新日志: <https://mootdx.readthedocs.io/zh_CN/latest/history/>\n\n运行环境\n--------\n\n-   操作系统: Windows / MacOS / Linux 都可以运行.\n-   Python: 3.8 以及以上版本.\n\n安装方法\n--------\n\n> 新手建议使用 `pip install -U 'mootdx[all]'` 安装\n\n### PIP 安装方法\n```shell\n\n# 包含核心依赖安装\npip install 'mootdx'\n\n# 包含命令行依赖安装, 如果使用命令行工具可以使用这种方式安装\npip install 'mootdx[cli]'\n\n# 包含所有扩展依赖安装, 如果不清楚各种依赖关系就用这个命令\npip install 'mootdx[all]'\n```\n\n### 升级安装\n\n```shell\npip install -U tdxpy mootdx\n```\n\n> 如果不清楚各种依赖关系就用这个命令 `pip install -U 'mootdx[all]'`\n\n使用说明\n--------\n\n> 以下只列举一些例子, 详细说明请查看在线文档: <https://www.mootdx.com>\n\n通达信离线数据读取\n\n```python\nfrom mootdx.reader import Reader\n\n# market 参数 std 为标准市场(就是股票), ext 为扩展市场(期货，黄金等)\n# tdxdir 是通达信的数据目录, 根据自己的情况修改\n\nreader = Reader.factory(market='std', tdxdir='C:/new_tdx')\n\n# 读取日线数据\nreader.daily(symbol='600036')\n\n# 读取分钟数据\nreader.minute(symbol='600036')\n\n# 读取时间线数据\nreader.fzline(symbol='600036')\n```\n\n通达信线上行情读取\n\n```python\nfrom mootdx.quotes import Quotes\n\n# 标准市场\nclient = Quotes.factory(market='std', multithread=True, heartbeat=True)\n\n# k 线数据\nclient.bars(symbol='600036', frequency=9, offset=10)\n\n# 指数\nclient.index(symbol='000001', frequency=9)\n\n# 分钟\nclient.minute(symbol='000001')\n```\n\n通达信财务数据读取\n\n```python\nfrom mootdx.affair import Affair\n\n# 远程文件列表\nfiles = Affair.files()\n\n# 下载单个\nAffair.fetch(downdir='tmp', filename='gpcw19960630.zip')\n\n# 下载全部\nAffair.parse(downdir='tmp')\n```\n\n加微信交流\n----------\n\n![](docs/img/IMG_2851.JPG)\n\n常见问题\n--------\n\nM1 mac 系统PyMiniRacer不能使用，访问:\n<https://github.com/sqreen/PyMiniRacer/issues/143>\n",
    'author': 'bopo',
    'author_email': 'ibopo@126.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://www.mootdx.com',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from abc import ABC
from pathlib import Path

from tdxpy.reader import TdxExHqDailyBarReader
from tdxpy.reader import TdxLCMinBarReader
from tdxpy.reader import TdxMinBarReader

from mootdx.contrib.compat import MooTdxDailyBarReader
from mootdx.utils import get_stock_market
from mootdx.utils import to_data


class Reader(object):
    @staticmethod
    def factory(market='std', **kwargs):
        """
        Reader 工厂方法

        :param market: std 标准市场, ext 扩展市场
        :param kwargs: 可变参数
        :return:
        """

        if market == 'ext':
            return ExtReader(**kwargs)

        return StdReader(**kwargs)


class ReaderBase(ABC):
    # 默认通达信安装目录
    tdxdir = 'C:/new_tdx'

    def __init__(self, tdxdir=None):
        """
        构造函数

        :param tdxdir: 通达信安装目录
        """

        if not Path(tdxdir).is_dir():
            raise Exception('tdxdir 目录不存在')

        self.tdxdir = tdxdir

    def find_path(self, symbol=None, subdir='lday', suffix=None, **kwargs):
        """
        自动匹配文件路径，辅助函数

        :param symbol:
        :param subdir:
        :param suffix:
        :return: pd.dataFrame or None
        """

        # 判断市场, 带#扩展市场
        if '#' in symbol:
            market = 'ds'
        # 通达信特有的板块指数88****开头的日线数据放在 sh 文件夹下
        elif symbol.startswith('88'):
            market = 'sh'
        else:
            # 判断是sh还是sz
            market = get_stock_market(symbol, True)

        # 判断前缀(市场是sh和sz重置前缀)
        if market.lower() in ['sh', 'sz']:
            symbol = market + symbol.lower().replace(market, '')

        # 判断后缀
        suffix = suffix if isinstance(suffix, list) else [suffix]

        # 调试使用
        if kwargs.get('debug'):
            return market, symbol, suffix

        # 遍历扩展名
        for ex_ in suffix:
            ex_ = ex_.strip('.')
            vipdoc = Path(self.tdxdir) / 'vipdoc' / market / subdir / f'{symbol}.{ex_}'

            if Path(vipdoc).exists():
                return vipdoc

        return None


class StdReader(ReaderBase):
    """股票市场"""

    def daily(self, symbol=None, **kwargs):
        """
        获取日线数据

        :param symbol: 证券代码
        :return: pd.dataFrame or None
        """
        symbol = Path(symbol).stem
        reader = MooTdxDailyBarReader()
        vipdoc = self.find_path(symbol=symbol, subdir='lday', suffix='day')

        result = reader.get_df(str(vipdoc)) if vipdoc else None
        return to_data(result, symbol=symbol, **kwargs)

    def minute(self, symbol=None, suffix=1, **kwargs):  # noqa
        """
        获取1, 5分钟线

        :param suffix: 文件前缀
        :param symbol: 证券代码
        :return: pd.dataFrame or None
        """
        symbol = Path(symbol).stem
        subdir = 'fzline' if str(suffix) == '5' else 'minline'
        suffix = ['lc5', '5'] if str(suffix) == '5' else ['lc1', '1']
        symbol = self.find_path(symbol, subdir=subdir, suffix=suffix)

        if symbol is not None:
            reader = TdxMinBarReader() if 'lc' not in symbol.suffix else TdxLCMinBarReader()
            return reader.get_df(str(symbol))

        return None

    def fzline(self, symbol=None):
        """
        分钟线数据

        :param symbol: 自定义板块股票列表, 类型 list
        :return: pd.dataFrame or Bool
        """
        return self.minute(symbol, suffix=5)

    def block_new(self, name: str = None, symbol: list = None, group=False, **kwargs):
        """
        自定义板块数据操作

        :param name: 自定义板块名称
        :param symbol: 自定义板块股票列表, 类型 list
        :param group:
        :return: pd.dataFrame or Bool
        """
        from mootdx.tools.customize import Customize

        reader = Customize(tdxdir=self.tdxdir)

        if symbol:
            return reader.create(name=name, symbol=symbol, **kwargs)

        return reader.search(name=name, group=group)

    def block(self, symbol='', group=False, **kwargs):
        """
        获取板块数据

        :param symbol:  板块文件
        :param group:   分组解析
        :return: pd.dataFrame or None
        """
        # from mootdx.block import BlockParse
        from mootdx.parse import BaseParse

        return BaseParse(self.tdxdir).parse(symbol, group=group, **kwargs)


class ExtReader(ReaderBase):
    """扩展市场读取"""

    def __init__(self, tdxdir=None):
        super(ExtReader, self).__init__(tdxdir)
        self.reader = TdxExHqDailyBarReader()

    def daily(self, symbol=None):
        """
        获取扩展市场日线数据

        :return: pd.dataFrame or None
        """

        vipdoc = self.find_path(symbol=symbol, subdir='lday', suffix='day')
        return self.reader.get_df(str(vipdoc)) if vipdoc else None

    def minute(self, symbol=None):
        """
        获取扩展市场分钟线数据

        :return: pd.dataFrame or None
        """

        if not symbol:
            return None

        vipdoc = self.find_path(symbol=symbol, subdir='minline', suffix=['lc1', '1'])
        return self.reader.get_df(str(vipdoc)) if vipdoc else None

    def fzline(self, symbol=None):
        """
        获取日线数据

        :return: pd.dataFrame or None
        """

        vipdoc = self.find_path(symbol=symbol, subdir='fzline', suffix='lc5')
        return self.reader.get_df(str(vipdoc)) if symbol else None



import datetime
import logging
import re
from pathlib import Path

import httpx
import pandas as pd
from tenacity import retry
from tenacity import stop_after_attempt
from tenacity import wait_fixed

from mootdx import get_config_path
from mootdx.cache import file_cache
from mootdx.consts import return_last_value
from mootdx.exceptions import MootdxModuleNotFoundError
from mootdx.logger import logger

JS_DECODE = (Path(__file__).parent / 'holiday.js').read_text(encoding='utf-8')


def holidays() -> pd.DataFrame:
    try:
        from py_mini_racer import py_mini_racer
    except (ImportError, ModuleNotFoundError):
        logging.warning('!!! 缺少依赖, 请使用次命令进行安装: pip install py_mini_racer')
        raise MootdxModuleNotFoundError('!!! 缺少依赖, 请使用次命令进行安装: pip install py_mini_racer')

    cache_file = get_config_path('caches/holidays.plk')

    @file_cache(filepath=cache_file, refresh_time=3600 * 24)
    @retry(wait=wait_fixed(2), retry_error_callback=return_last_value, stop=stop_after_attempt(5))
    def _holidays() -> pd.DataFrame:

        logger.debug('调用远程接口')
        client = httpx.Client(verify=False)

        url = 'https://finance.sina.com.cn/realstock/company/klc_td_sh.txt'
        res = client.get(url)

        js_code = py_mini_racer.MiniRacer()
        js_code.eval(JS_DECODE)

        # 执行js解密代码
        dict_list = js_code.call('d', res.text.split('=')[1].split(';')[0].replace('"', ''))

        temp_df = pd.DataFrame(dict_list)
        temp_df.columns = ['date']
        temp_df['date'] = pd.to_datetime(temp_df['date']).dt.date

        temp_list = temp_df['date'].to_list()
        temp_list.append(datetime.date(1992, 5, 4))  # 是交易日但是交易日历缺失该日期
        temp_list.sort()

        temp_df = pd.DataFrame(temp_list, columns=['date'])
        temp_df['year'] = pd.DatetimeIndex(temp_df['date']).year

        return temp_df

    result = _holidays()

    if result.empty:
        Path(cache_file).unlink(missing_ok=True)
        return pd.DataFrame([])

    return result


def holiday2(date: str = None) -> pd.DataFrame:
    """交易日历-历史数据
    :return: 交易日历
    :rtype: pandas.DataFrame
    """

    temp_df = holidays()

    if date:
        try:
            date = datetime.datetime.strptime(date, '%Y-%m-%d').date()
        except ValueError:
            date = datetime.datetime.now().date()

        return temp_df[temp_df['date'] == date]

    return temp_df


def holiday(date=None, format_=None, country=None, result=False):
    format_ = format_ if format_ else '%Y-%m-%d'
    country = country if country else '中国'

    try:
        if date:
            date = datetime.datetime.strptime(date, format_).date()
        else:
            date = datetime.datetime.now().date()
    except ValueError as ex:
        logger.error('日期或者日期格式错误!')
        return None

    df = _holiday()

    if country not in list(set(df['国家'].values)):
        logger.error(f'没有该国家`{country}`的交易日数据')
        return None

    df = df[df['国家'] == country]
    df = df[df.index.isin([date])]

    if result:
        return df

    logger.debug(date.weekday())

    return not df.empty or date.weekday() >= 5


@file_cache(filepath=get_config_path('caches/holiday.plk'), refresh_time=3600 * 24)
def _holiday():
    logger.debug('调用远程接口')
    res = httpx.get('https://www.tdx.com.cn/url/holiday/')
    res.encoding = 'gbk'

    ret = re.findall(r'<textarea id="data" style="display:none;">([\s\w\W]+)</textarea>', res.text, re.M)[0].strip()
    day = [d.split('|')[:4] for d in ret.split('\n')]

    df = pd.DataFrame(day, columns=['日期', '节日', '国家', '交易所'], dtype=str)
    df.index = pd.to_datetime(df['日期'].astype('str'), format='%Y%m%d')

    if df.empty:
        Path(get_config_path('caches/holiday.plk')).unlink(missing_ok=True)
        return pd.DataFrame([])

    return df


def holiday_(date=None, format_=None, country=None):
    return holiday(date=date, format_=format_, country=country, result=True)
